/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package marg.test;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.ServiceLoader;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import marg.model.plugin.XMLParsePlugin;

/**
 *
 * @author Daniel
 */
public class OldTestPluginManager {

    ArrayList<XMLParsePlugin> xmlParsePlugins = new ArrayList<XMLParsePlugin>();
    private static ServiceLoader<XMLParsePlugin> xmlParsePluginLoader = ServiceLoader.load(XMLParsePlugin.class);

    public OldTestPluginManager() {
        try {
            initParsePlugins();
        } catch (Exception ex) {
            Logger.getLogger(OldTestPluginManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void initParsePlugins() throws Exception {
        Class[] classes = getClasses("marg.model.plugin");
        for (Class cl : classes) {
            System.out.println("Found Class: "+ cl);
            Class[] interfaces = cl.getInterfaces();
            for (Class aCl : interfaces) {
                if (aCl.equals(XMLParsePlugin.class)) {
                    System.out.println("^ implements XMLParsePlugin");
                    JOptionPane.showMessageDialog(null, "Found Class: "+ cl +" which implements XMLParsePlugin");
                }
            }
        }
    }

    public static void main(String[] args) throws Exception {
        OldTestPluginManager plugMan = new OldTestPluginManager();
        plugMan.initParsePlugins();
    }

    /**
     * Scans all classes accessible from the context class loader which belong to the given package and subpackages.
     *
     * @param packageName The base package
     * @return The classes
     */
    private static Class[] getClasses(String packageName)
            throws ClassNotFoundException, IOException {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        assert classLoader != null;
        String path = packageName.replace('.', '/');
        Enumeration<URL> resources = classLoader.getResources(path);
        List<File> dirs = new ArrayList<File>();
        while (resources.hasMoreElements()) {
            URL resource = resources.nextElement();
            dirs.add(new File(resource.getFile().replace("%20", " ")));
        }
        ArrayList<Class> classes = new ArrayList<Class>();
        for (File directory : dirs) {
            classes.addAll(findClasses(directory, packageName));
        }
        return classes.toArray(new Class[classes.size()]);
    }

    /**
     * Recursive method used to find all classes in a given directory and subdirs.
     * @param directory   The base directory
     * @param packageName The package name for classes found inside the base directory
     * @return The classes
     */
    private static List<Class> findClasses(File directory, String packageName) {
        List<Class> classes = new ArrayList<Class>();
        if (!directory.exists()) {
            return classes;
        }
        File[] files = directory.listFiles();
        for (File file : files) {
            if (file.isDirectory()) {
                assert !file.getName().contains(".");
                classes.addAll(findClasses(file, packageName + "." + file.getName()));
            } else if (file.getName().endsWith(".class")) {
                try {
                    classes.add(Class.forName(packageName + '.' + file.getName().substring(0, file.getName().length() - 6)));
                } catch (ClassNotFoundException ex) {
                    System.out.println("Class not found: "+ file);
                }
            }
        }
        return classes;
    }
}
